package vif

  1. Overview
  2. Docs

Module Route.UtilsSource

Sourceval map_snd : ('a -> 'b) -> ('c * 'a) -> 'c * 'b
Sourceval intersperse : 'a -> 'a list -> 'a list
Sourceval find_idx : ('a -> int) -> 'a -> 'a list -> int

Offset of el in l, given the function count. Used to get the first regexp group at a given place.

Sourceval build_permutation : int -> ('a -> int) -> 'a list -> 'a list -> int array

if l' ∈ l then build_permutation offset count l l' builds a mapping: index in l => offset in l'. Offsets are computed respecting offset and count.