package vif

  1. Overview
  2. Docs

Module Vif.QueriesSource

A request not only has content but also parameters (queries) that can be specified via the URI. This module allows you to extract the values specified in the URI to the user when managing the request.

Sourceval exists : ('c, 'a) Request.t -> string -> bool

exists req query checks whether the query information has been provided by the user via the URI.

Sourceval get : ('c, 'a) Request.t -> string -> string list

get req query returns the values associated with the key query given in the URI.