Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Vif.RouteSourceget uri describes a route which matches a GET request with the given path uri. a GET request does not have any contents.
head uri describes a route which matches a HEAD request with the given path uri. A HEAD request does not have any contents.
delete uri describes a route which matches a DELETE request with the given path uri. A DELETE request does not have any contents.
post ty uri describes a route which matches a POST request with a certain Content-Type described by ty and with the given path uri.
put ty uri describes a route which matches a PUT request with a certain Content-Type described by ty and with the given path uri.