package vif

  1. Overview
  2. Docs

v0.0.1~beta2 (2026-02-23)

  • Add Vif.Queries.all (@reynir, #5)
  • Improve examples and README.md (@dinosaure, @reynir, #7, #9)
  • Rename Vif.Uri.path to Vif.Uri.path (@dinosaure, @reynir, #8)
  • Fix errors when we parse multipart/form-data (@dinosaure, spotted by @yomimono, #14, #22)
  • Catch exceptions from URI converters (@reynir, @dinosaure, #12)
  • Add ?etag argument for files (@reynir, #16)
  • Be able to have a regular expression per HTTP methods (@reynir, #18)
  • Add Vif.Uri.execp and Vif.Uri.extract (@reynir, #20)
  • Exclude ? from Vif.Uri.string `Path (@reynir, #23)
  • Add Vif.Response.with_text (@dinosaure, #24, #37)
  • Fix the support of MacOS (@vvvvv, @dinosaure, spotted by @voodoos, #25)
  • Introduce vifu, a unikernel compatible version of vif (@dinosaure, #28)
  • Fix Vif.Response.with_file (@dinosaure, #29)
  • Fix a double-close on Vif.Response.with_json and how the function is exposed (@dinosaure, #30, #37)
  • Let the user to define a log reporter (@voodoos, @dinosaure, #32)
  • Fix the support of h2 protocol and which field we should generate (@dinosaure, #33)
  • Use flux (@dinosaure, #34)
  • Fix memory leak with logs (@dinosaure, #38, #39)
  • Update to tyre.1.0.0 (@dinosaure, @EmileTrotignon, #40)
  • Remove the usage of ppx in our tests (@dinosaure, #42)
  • Verify the TLS configuration given (and its ALPN protocols) (@dinosaure, spotted by @voodoos, #41)
  • Provide Vif.Response.with_stream (@dinosaure, #43)
  • Remove deprecated jsonm dependency (@dinosaure, #45)
  • Give the possibility for the user to initialize RNG (@dinosaure, requested by @voodoos, #46)
  • Remove pin-depends (@hannesm, @dinosaure, #48)

v0.0.1~beta1 (2025-08-14)

  • First release of Vif