Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Route.UtilsSourceOffset of el in l, given the function count. Used to get the first regexp group at a given place.
if l' ∈ l then build_permutation offset count l l' builds a mapping: index in l => offset in l'. Offsets are computed respecting offset and count.